-- == SRB2Randomizer: Character randomization ==
-- Based off of toaster's character diceroll

randomizer.cvars.characters = CV_RegisterVar({
	name = "randomizer_characters",
	flags = CV_NETVAR|CV_SHOWMODIF,
	PossibleValue = {
		Off = 0,
		Replace = 1,
		Random = 2,
	},
	defaultvalue = "Replace",
})

randomizer.newHelpEntry(
	"characters",
	"<integer/string>",
	"Changes character randomization style.",
	{
		"Changes character randomization style. Characters will not be randomized in Record Attack. Options are...",
		" * \130Off (0):\128 Characters will not be randomized -- you can use whatever character you’d like.",
		" * \130Replace (1):\128 Character randomization is per level. Each level gets its own pre-defined character. This is more fair for races. (default option)",
		" * \130Random (2):\128 Character randomization is per spawn. You can get rid of a bad roll by retrying, or waste a good roll by dying.",
	},
	false
)

randomizer.cvars.noforcechar = CV_RegisterVar({
	name = "randomizer_noforcechar",
	flags = CV_NETVAR|CV_SHOWMODIF,
	PossibleValue = CV_OnOff,
	defaultvalue = "On",
})

randomizer.newHelpEntry(
	"noforcechar",
	"<boolean>",
	"Override forced character settings.",
	{
		"Decides how to handle maps that force you to a specific character (such as SRB1 Remake). Options are...",
		" * \130Off (0):\128 Always use the map's forced character.",
		" * \130On (1):\128 If characters are being randomized, then use the randomizer's choice instead of the map's. May cause unwinnable scenarios if a map expects that certain character. (default option)",
	},
	false
)

randomizer.cvars.prefcolors = CV_RegisterVar({
	name = "randomizer_prefcolors",
	flags = CV_SHOWMODIF,
	PossibleValue = CV_OnOff,
	defaultvalue = "On",
	-- Choose whenever or not to randomize each character's prefcolor in SP/splitscreen mode.
})

randomizer.newHelpEntry(
	"prefcolors",
	"<boolean>",
	"Randomize character colors in 1 Player and Splitscreen.",
	{
		"Choose to randomize each character's preferred color in singleplayer and splitscreen modes. Has no effect in netgames. Options are...",
		" * \130Off (0):\128 Use the character's canon color.",
		" * \130On (1):\128 Give every character a new color. (default option)",
	},
	true
)

local function skinChange(player, num)
	if (R_SkinUsable(player, num) == false)
		return
	end

	R_SetPlayerSkin(player, num)
	COM_BufInsertText(player, "skin " + skins[num].name) -- This needs to be done to fix the MP character select menu.

	if not (netgame) -- Presumably, you'd want the skincolour to stay constant - since it can help identify a player, and would be a pain to set each map.
		local prefcolor = skins[num].prefcolor

		if (randomizer.cvars.prefcolors.value)
			local availableskincolors = {}

			for i = 1,(#skincolors-1)
				if (skincolors[i].accessible == true)
					table.insert(availableskincolors, i)
				end
			end

			prefcolor = availableskincolors[(randomizer.scramble(randomizer.seed + num + skins[num].prefcolor) % #availableskincolors) + 1]
		end

		player.skincolor = prefcolor
		player.mo.color = prefcolor
		COM_BufInsertText(player, "color "..prefcolor) -- Ditto.
	end
end

addHook("ThinkFrame", function(mo)
	if (randomizer.seed == nil or randomizer.rv == nil)
		return
	end

	if (randomizer.cvars.characters.value == 0) -- != 1
		return
	end

	if (modeattacking)
		return
	end

	for p in players.iterate()
		if (p.randomizer == nil)
			continue
		end

		if (p.mo and p.mo.valid)
			if (randomizer.cvars.noforcechar.value == 0)
			and (mapheaderinfo[gamemap].forcecharacter)
				p.randomizer.skin = #(skins[mapheaderinfo[gamemap].forcecharacter])
			elseif (p.randomizer.skin == nil)
			or (p.randomizer.prevmap == nil)
			or (p.randomizer.prevmap != gamemap)
				local randomvalue = randomizer.scramble(randomizer.scramble(randomizer.seed) + randomizer.scramble((#p+1) + gamemap))
				p.randomizer.skin = (randomvalue % #skins)
			end

			p.randomizer.prevmap = gamemap

			if (#skins[p.mo.skin] != p.randomizer.skin)
				skinChange(p, p.randomizer.skin)
			end
		end
	end
end)

addHook("PlayerSpawn", function(p)
	if (randomizer.seed == nil or randomizer.rv == nil)
		return
	end

	randomizer.debugPrint(" == Per-spawn character randomization for player " .. #p .. " == ")

	if (randomizer.cvars.characters.value != 2)
		randomizer.debugPrint("Turned off, skipping")
		return
	end

	if (modeattacking)
		randomizer.debugPrint("Record Attack, skipping")
		return
	end

	if (randomizer.cvars.noforcechar.value == 0)
	and (mapheaderinfo[gamemap].forcecharacter)
		p.randomizer.skin = #(skins[mapheaderinfo[gamemap].forcecharacter])
	else
		local randomvalue = randomizer.scramble(randomizer.scramble(randomizer.seed) + randomizer.scramble((#p+1) + gamemap + leveltime))
		p.randomizer.skin = (randomvalue % #skins)
	end

	p.randomizer.prevmap = gamemap
	skinChange(p, p.randomizer.skin)
end)
