-- == SRB2Randomizer: Enemy randomization ==
-- Changes enemy types in the levels around

randomizer.cvars.enemies = CV_RegisterVar({
	name = "randomizer_enemies",
	flags = CV_NETVAR|CV_SHOWMODIF,
	PossibleValue = {
		Off = 0,
		On = 1,
		Zone = 2,
		Full = 3,
	},
	defaultvalue = "On",
})

randomizer.newHelpEntry(
	"enemies",
	"<integer/string>",
	"Changes enemy randomization style.",
	{
		"Changes enemy randomization style. Options are...",
		" * \130Off (0):\128 Enemies will not be randomized.",
		" * \130On (1):\128 Enemy types will be randomized. The number of unique enemies per zone is kept in-tact. (default option)",
		" * \130Zone (2):\128 Enemies will be properly themed to the level you're in, by only swapping around enemy types in the current level. The most tame option.",
		" * \130Full (3):\128 Any type of enemy can appear in any spot. The most chaotic option.",
	},
	false
)

local function replaceEnemies()
	local enemytypes = {}
	local enemyspots = {}
	local canRandomize = false

	if (randomizer.cvars.enemies.value == 1)
		randomizer.debugPrint("Normal enemy randomize")
		local levelenemytypes = {}

		for mo in mobjs.iterate()
			if not (mo and mo.valid)
				continue
			end

			if (mo.flags & MF_ENEMY)
				table.insert(enemyspots, mo)

				local nodupes = true
				if (#levelenemytypes > 0)
					for i = 1,#levelenemytypes
						if (levelenemytypes[i] == mo.type)
							nodupes = false
							break
						end
					end
				end

				if (nodupes)
					table.insert(levelenemytypes, mo.type)
				end
			end
		end

		canRandomize = (#levelenemytypes > 0) -- Only randomize when there's enough types

		if (canRandomize == true)
			local allenemytypes = {}

			for i = 0,#mobjinfo-1
				if (mobjinfo[i].flags & MF_ENEMY)
				and (mobjinfo[i].doomednum != -1) -- make sure it's intended to be placeable
					table.insert(allenemytypes, i)
				end
			end

			for i = 1,#levelenemytypes
				local key = (randomizer.rv % #allenemytypes) + 1
				randomizer.rv = randomizer.scramble(randomizer.rv)
				table.insert(enemytypes, allenemytypes[key])
				table.remove(allenemytypes, key)
			end
		end
	elseif (randomizer.cvars.enemies.value == 2)
		randomizer.debugPrint("Zone enemy randomize")
		-- Only randomize per zone
		for mo in mobjs.iterate()
			if not (mo and mo.valid)
				continue
			end

			if (mo.flags & MF_ENEMY)
				table.insert(enemyspots, mo)

				local nodupes = true
				if (#enemytypes > 0)
					for i = 1,#enemytypes
						if (enemytypes[i] == mo.type)
							nodupes = false
							break
						end
					end
				end

				if (nodupes)
					table.insert(enemytypes, mo.type)
				end
			end
		end

		canRandomize = (#enemytypes > 1) -- Only randomize when there's enough types
	elseif (randomizer.cvars.enemies.value == 3)
		randomizer.debugPrint("Full enemy randomize")
		-- Full random
		canRandomize = true

		for i = 0,#mobjinfo-1
			if (mobjinfo[i].flags & MF_ENEMY)
			and (mobjinfo[i].doomednum != -1) -- make sure it's intended to be placeable
				table.insert(enemytypes, i)
			end
		end

		for mo in mobjs.iterate()
			if not (mo and mo.valid)
				continue
			end

			if (mo.flags & MF_ENEMY)
				table.insert(enemyspots, mo)
			end
		end
	else
		assert(false, "Bad enemy randomizer option")
		return
	end

	if (canRandomize == false)
		return
	end

	for i = 1,#enemyspots
		if (enemyspots[i] and enemyspots[i].valid)
			local mo = enemyspots[i]

			-- Because of how crazy some enemy types can be, especially Lua ones,
			-- we're spawning new enemies then removing the old ones later

			local newtype = enemytypes[(randomizer.rv % #enemytypes) + 1]

			randomizer.rv = randomizer.scramble(randomizer.rv)

			if (mo.type != newtype)
				local newflip = (mo.flags2 & MF2_OBJECTFLIP)
				local newz = mo.z
				if (newflip)
					newz = $1 + mo.height - mobjinfo[newtype].height
				end

				mo.flags = $1|MF_NOCLIPTHING|MF_NOTHINK

				if (mo.type == MT_YELLOWSHELL)
					-- Yellow Spring Shells are considered important for progression and normally drop their spring.
					local springz = mo.z
					if (newflip)
						springz = $1 + mo.height - mobjinfo[MT_YELLOWSPRING].height
					end

					local spring = P_SpawnMobj(mo.x, mo.y, springz, MT_YELLOWSPRING)
					if (newflip)
						spring.eflags = $1|MFE_VERTICALFLIP
						spring.flags2 = $1|MF2_OBJECTFLIP
					end
				end

				local newmo = P_SpawnMobj(mo.x, mo.y, newz, newtype)
				newmo.angle = mo.angle
				if (newflip)
					newmo.eflags = $1|MFE_VERTICALFLIP
					newmo.flags2 = $1|MF2_OBJECTFLIP
				end

				--[[
				if (mo.spawnpoint and mo.spawnpoint.valid)
					newmo.spawnpoint = mo.spawnpoint
					newmo.spawnpoint.type = mobjinfo[newmo.type].doomednum
					newmo.spawnpoint.mobj = newmo
				end
				--]]

				P_RemoveMobj(enemyspots[i])
			end
		end
	end
end

addHook("MapLoad", function()
	if (randomizer.seed == nil or randomizer.rv == nil)
		return
	end

	randomizer.debugPrint(" == Enemy randomization == ")

	if (randomizer.cvars.enemies.value != 0)
		replaceEnemies()
	end
end)
