-- == SRB2Randomizer: Help commands ==
-- Should always be the last file in the list,
-- so that the inital print isn't obscured by everything loading.

local prefix = "randomizer_"

local function printDetailedHelp(p, help)
	CONS_Printf(p, "")
	CONS_Printf(p, "\130"..prefix+help.name.." "..help.args..":")
	for i = 1,#(help.detailed)
		CONS_Printf(p, "  "..help.detailed[i])
	end
	CONS_Printf(p, "")
end

COM_AddCommand("randomizer_help", function(p, arg)
	if (arg != nil)
		-- Individual help
		local input = string.gsub(arg, prefix, "", 1)

		-- Server commands
		for _,v in pairs(randomizer.help.server)
			if (input == v.name)
				printDetailedHelp(p, v)
				return
			end
		end

		-- Client commands
		for _,v in pairs(randomizer.help.client)
			if (input == v.name)
				printDetailedHelp(p, v)
				return
			end
		end

		CONS_Printf(p, "No command/variable called \""..prefix+input.."\".")
	else
		-- Global help
		CONS_Printf(p, "")
		CONS_Printf(p, "\130   * SERVER OPTIONS *")
		CONS_Printf(p, "")

		for _,v in pairs(randomizer.help.server)
			if (v.tag)
				CONS_Printf(p, " * \130"..prefix+v.name..":\128 "..v.tag)
			end
		end

		CONS_Printf(p, "")
		CONS_Printf(p, "\130   * CLIENT OPTIONS *")
		CONS_Printf(p, "")

		for _,v in pairs(randomizer.help.client)
			if (v.tag)
				CONS_Printf(p, " * \130"..prefix+v.name..":\128 "..v.tag)
			end
		end

		CONS_Printf(p, "")
		CONS_Printf(p, "Type \130randomizer_help <command/variable name>\128 for more information on a specific option.")
		CONS_Printf(p, "")
	end
end, 0)

randomizer.newHelpEntry(
	"help",
	"[<command/variable name>]",
	nil, -- Don't show in global help list
	{"You clearly already know how to use this command."},
	false
)

print(
	"",
	"\130   * SRB2Randomizer (v2) *",
	"",
	" * To get started, type \130randomizer_setseed [<seed>]\128 in the console to start your randomizer!",
	" * Toggle the various options using the console, includes settings such as \130randomizer_characters\128, \130randomizer_levelorder\128, \130randomizer_monitors\128, etc.",
	" * Any added custom characters, levels, monitors, or enemies will be added to the randomization pool automatically!",
	" * Both the seed and a hash of the settings are displayed on the Rankings/Scores HUD. Use this to prevent cheating during a speed-run race.",
	" * For more information, type \130randomizer_help\128.",
	""
)
