-- == SRB2Randomizer: Level order randomization ==
-- Changes map order of the whole campagin

randomizer.cvars.levelorder = CV_RegisterVar({
	name = "randomizer_levelorder",
	flags = CV_NETVAR|CV_SHOWMODIF,
	PossibleValue = {
		Off = 0,
		-- No level randomization.
		-- Use the standard level order.
		All = 1,
		-- The randomized level order will contain all compatible levels.
		NoSecrets = 2,
		-- The randomized level order will only use levels from the standard campaign.
		-- Removes Aerial Garden, Azure Temple, Pipe Towers, Christmas Chime, etc.
		-- Packs that use unlockables for progression (like SUGOI) will give you a tiny map pool with this option set.
		NoAlternative = 3,
		-- The randomized level order will use everything except "alternative" gametypes. (2D Mode and NiGHTS)
	},
	defaultvalue = "NoSecrets",
})

randomizer.newHelpEntry(
	"levelorder",
	"<integer/string>",
	"Changes level order randomization style.",
	{
		"Changes level order randomization style. Options are...",
		" * \130Off (0):\128 Level order will not be randomized. Could be good for level packs that have multiple exits, or simply if you prefer a tamer experience.",
		" * \130All (1):\128 Level order will be randomized, using all compatible levels that are added.",
		" * \130NoSecrets (2):\128 Level order will be randomized using only levels from the standard campaign. Removes Aerial Garden, Azure Temple, Pipe Towers, Christmas Chime, etc.",
		"Packs that use unlockables for progression (like SUGOI) will give you an insanely tiny map pool with this option. (default option)",
		" * \130NoAlternative (3):\128 Level order will be randomized using everything except for maps that use \"alternative\" gametypes (2D Mode and NiGHTS).",
	},
	false
)

--[[
randomizer.cvars.ssorder = CV_RegisterVar({
	name = "randomizer_ssorder",
	flags = CV_NETVAR|CV_SHOWMODIF,
	PossibleValue = CV_OnOff,
	defaultvalue = "On",
	func = forceOrderUpdate,
	-- Decide whenever or not to randomize Special Stage order.
	-- (Disabled because of hardcoded Special Stage progression interfering.)
})
--]]

local firstMapInitalized = false

local function initFirstMap()
	if (firstMapInitalized)
		return
	end

	if (#randomizer.levellist <= 0)
		randomizer.debugPrint("No level list, not changing map.")
		return
	end

	if (server != nil)
		print("Level list has changed, going to first map in rotation...")
		firstMapInitalized = true

		--if (gamemap != randomizer.levellist[1])
			G_ExitLevel(randomizer.levellist[1], 2)
		--end
	end
end

function randomizer.updateLevelList()
	randomizer.debugPrint(" == Level list randomization == ")

	if (randomizer.cvars.levelorder.value == 0)
		randomizer.levellist = {}
		firstMapInitalized = false
		randomizer.debugPrint("Level list has been emptied.")
		return
	end

	local availablelevels = {}
	
	-- Initialize the new level order.
	randomizer.levellist = {}
	firstMapInitalized = false

	local tol = TOL_SP
	if (multiplayer or netgame)
		tol = TOL_COOP
	end

	for i = 1,1035
		if (G_IsSpecialStage(i))
			continue
		end

		if (mapheaderinfo[i] == nil)
			continue
		end

		if (mapheaderinfo[i].typeoflevel & tol)
		and (randomizer.cvars.levelorder.value != 0)
			if (i == tutorialmap or i == titlemap)
				continue
			elseif (randomizer.cvars.levelorder.value != 1)
				-- No "alternative gamemode" maps
				if (mapheaderinfo[i].typeoflevel & TOL_2D)
				or (mapheaderinfo[i].typeoflevel & TOL_NIGHTS)
					continue
				end

				if (randomizer.cvars.levelorder.value == 2)
					-- No secret maps
					if (mapheaderinfo[i].unlockrequired != -1)
						continue
					end
				end
			end

			table.insert(availablelevels, i)
		end
	end

	local randomvalue = randomizer.seed
	local header = mapheaderinfo[map]
	randomvalue = randomizer.scramble(randomvalue)

	while (#availablelevels > 0)
		local i = (randomvalue % #availablelevels) + 1
		header = mapheaderinfo[availablelevels[i]]
		table.insert(randomizer.levellist, availablelevels[i])
		randomizer.debugPrint("Added level " .. availablelevels[i] .. " to rotation")
		randomvalue = randomizer.scramble(randomvalue)
		table.remove(availablelevels, i)
	end

	initFirstMap()
end

addHook("ThinkFrame", function()
	if (randomizer.seed == nil or randomizer.rv == nil)
		return
	end

	if (randomizer.cvars.levelorder.value == 0)
		return
	end

	if (firstMapInitalized == false)
		if (leveltime)
			initFirstMap()
		end
		return
	end

	local exiting = false
	for p in players.iterate()
		if (p.exiting > 0)
			-- This check is here to prevent going
			-- to the next level on Game Over
			exiting = true
			break
		end
	end

	if (exiting == false)
		return
	end

	if (#randomizer.levellist > 0)
		local newnextlevel = nil

		for i = 1,#randomizer.levellist
			if (gamemap == randomizer.levellist[i])
				if (randomizer.levellist[i+1] == nil)
					newnextlevel = 1103 -- Ending
				else
					newnextlevel = randomizer.levellist[i+1]
				end

				break
			end
		end

		if (newnextlevel != nil)
			G_SetCustomExitVars(newnextlevel)
		end
	end
end)
