-- == SRB2Randomizer: Music randomization ==
-- Changes all music tracks around

randomizer.cvars.music = CV_RegisterVar({
	name = "randomizer_music",
	flags = CV_SHOWMODIF,
	PossibleValue = {
		Off = 0,
		-- No music randomization.
		-- Use the original music for each level.
		Replace = 1,
		-- Music will get a consistent replacement per seed.
		-- Ex: the boss music will get replaced with the same song each time.
		Random = 2,
		-- Roll RNG for music choices on every level load.
		-- You will hear a different music track every time you reload a level.
	},
	defaultvalue = "Replace",
})

randomizer.newHelpEntry(
	"music",
	"<integer/string>",
	"Changes music randomization style.",
	{
		"Changes music randomization style. Options are...",
		" * \130Off (0):\128 Normal music.",
		" * \130Replace (1):\128 Music will get a consistent replacement per seed. (default option)",
		" * \130Random (2):\128 Change the music choice on every level load. You will hear a different music track each time you die.",
	},
	true
)

local musiclist = {
	-- Main game
	"GFZ1",
	"GFZ2",
	"GFZALT",
	"THZ1",
	"THZ2",
	"THZALT",
	"DSZ1",
	"DSZ2",
	"CEZ1",
	"CEZ2",
	"CEZR",
	"ACZ1",
	"ACZ2",
	"RVZ1",
	"RVZ2",
	"RVZALT",
	"ERZ1",
	"ERZ2",
	"BCZ1",
	"BCZ2",

	-- Bosses
	"VSBOSS",
	"VSFANG",
	"VSMETL",
	"VSBRAK",
	"VSALT",
	"VSAGZ",

	-- Special Stages
	"SPEC1",
	"SPEC2",
	"SPEC3",
	"SPEC4",
	"SPEC5",
	"SPEC6",
	"SPEC7",
	"SPEC8",
	"OSPEC",
	"OSPEC2",

	-- Extras
	"FHZ",
	"HAUNTD",
	"AGZ",
	"AGZALT",
	"ATZ",
	"CCZ",
	"DHZ",
	"APZ1",
	"APZ2",
	"MARIO1",
	"MARIO2",

	-- Multiplayer
	"MP_SUN",
	"MP_DES",
	"MP_GHZ",
	"MP_ICE",
	"MP_DOM",
	"MP_MED",
	"MP_WTR",
	"MP_WT2",
	"MP_AIR",
	"MP_SUN",
	"MP_GLZ",
	"MP_GL2",
	"MP_MIN",
	"MP_COA",
	"MP_RAI",
	"MP_FIR",
	"MP_FOR",
	"MP_MRZ",
	"MP_0RI",
	"MP_FR2",

	-- Misc
	"_INTRO",
	"OINTRO",
	"TUTORL",
	"_CHSEL",
	"_INTER",
	"_RECAT",
	"_NITAT",
	"_CREDS",
	"BHZ",
	"DCZ",
	"DISCO",

	-- RANDOMIZER EXLCUSIVE MUSIC
	-- (Funny stuff from old versions)
	"SRB1A",
	"SRB1B",
	"SRB1C",
	"SRB1D",
	"SRB1E",
	"SRB1F",
	"SRB1G",
	"SRB1H",
	"SRB1I",
	"SRB1J",
	"SRB1K",
	"SRB1L",
	"SRB1M",
	"SRB1N",
	"SRB1O",
	"SRB1P",
	"SRB1Q",
	"SRB1R",
	"SRB1S",
	"SRB1T",
	"SRB1U",
	"SRB1V",
	"SRB1W",

	--[[ The xmas music was fucking MASSIVE
	"XMASA",
	"XMASB",
	"XMASC",
	"XMASD",
	"XMASE",
	"XMASF",
	"XMASG",
	"XMASH",
	"XMASI",
	--]]

	"CEZ3",
	"MKB3",
	"CREDIT",
	"METALH",
	"ZIM",
	"REYNTV",
	--"SPACE",
	"EVILFL",
}

local savedmusname = nil
local lastmap = nil

local function changeEveryonesMusic()
	-- New behavior
	if (randomizer.cvars.music.value != 0)
		for p in players.iterate()
			if (p.bot)
				continue
			end

			COM_BufInsertText(p, "tunes " + savedmusname)
		end
	end
end

local function setRandomizedMusic()
	local v = randomizer.seed

	if (randomizer.cvars.music.value == 2)
		v = $1 + gamemap
	end

	for i = 1,#mapmusname
		local b = mapmusname:byte(i,i)
		local mul = i

		if (i == #mapmusname)
			mul = 1
		end

		if (i & 1)
			v = $1 - (b * i)
		else
			v = $1 + (b * i)
		end
	end

	v = randomizer.scramble(v)

	savedmusname = musiclist[(v % #musiclist) + 1]
	changeEveryonesMusic()
end

local function addSong(songname)
	if (songname == nil or #songname <= 0)
		return
	end

	for i = 1,#musiclist
		if (songname == musiclist[i])
			return
		end
	end

	randomizer.debugPrint("Added song \"" .. songname .. "\"")
	table.insert(musiclist, songname)
end

local function scrapeForMusic()
	for i = 1,1035
		if (mapheaderinfo[i] == nil)
			continue
		end

		addSong(mapheaderinfo[i].musname)
		--addSong(mapheaderinfo[i].musintername)
		--addSong(mapheaderinfo[i].muspostbossname)
	end
end

addHook("MapChange", function(map)
	if (server == nil)
		lastmap = nil
	end

	if (map != lastmap) or (randomizer.cvars.music.value == 2)
		savedmusname = nil
		scrapeForMusic()
	end

	lastmap = map
	--setRandomizedMusic()
end)

addHook("ThinkFrame", function()
	if (randomizer.seed == nil or randomizer.rv == nil)
		return
	end

	if (randomizer.cvars.music.value == 0)
		return
	end

	if (savedmusname == nil or mapmusname != savedmusname)
		setRandomizedMusic()
	end
end)

addHook("NetVars", function(net)
	savedmusname = net(savedmusname)
	lastmap = net(lastmap)

	changeEveryonesMusic()
end)
